@@ -18,7 +18,9 @@
 package boot
 
 import (
+	"github.com/apache/skywalking-rover/pkg/accesslog"
 	"github.com/apache/skywalking-rover/pkg/core"
+	"github.com/apache/skywalking-rover/pkg/logger"
 	"github.com/apache/skywalking-rover/pkg/module"
 	"github.com/apache/skywalking-rover/pkg/process"
 	"github.com/apache/skywalking-rover/pkg/profiling"
@@ -26,7 +28,9 @@ import (
 
 func init() {
 	// register all active module
+	module.Register(logger.NewModule())
 	module.Register(core.NewModule())
 	module.Register(process.NewModule())
 	module.Register(profiling.NewModule())
+	module.Register(accesslog.NewModule())
 }
