@@ -22,6 +22,7 @@ import (
 	"context"
 	"fmt"
 	"os"
+	"regexp"
 	"strconv"
 	"strings"
 	"time"
@@ -40,6 +41,7 @@ import (
 
 	v3 "skywalking.apache.org/repo/goapi/collect/ebpf/profiling/process/v3"
 
+	"github.com/apache/skywalking-rover/pkg/core"
 	"github.com/apache/skywalking-rover/pkg/logger"
 	"github.com/apache/skywalking-rover/pkg/process/api"
 	"github.com/apache/skywalking-rover/pkg/process/finders/base"
@@ -48,10 +50,13 @@ import (
 
 var log = logger.GetLogger("process", "finder", "kubernetes")
 
+var kubepodsRegex = regexp.MustCompile(`cri-containerd-(?P<Group>\w+)\.scope`)
+
 type ProcessFinder struct {
 	conf *Config
 
 	// runtime
+	clusterName  string
 	manager      base.ProcessManager
 	ctx          context.Context
 	cancelCtx    context.CancelFunc
@@ -60,25 +65,26 @@ type ProcessFinder struct {
 
 	// k8s clients
 	k8sConfig *rest.Config
-	cli       *kubernetes.Clientset
 	registry  *Registry
+	CLI       *kubernetes.Clientset
 
 	// runtime config
 	namespaces []string
 }
 
 func (f *ProcessFinder) Init(ctx context.Context, conf base.FinderBaseConfig, manager base.ProcessManager) error {
+	f.clusterName = manager.GetModuleManager().FindModule(core.ModuleName).(core.Operator).ClusterName()
 	k8sConf, cli, err := f.validateConfig(ctx, conf.(*Config))
 	if err != nil {
 		return err
 	}
 	f.conf = conf.(*Config)
 	f.k8sConfig = k8sConf
-	f.cli = cli
+	f.CLI = cli
 
 	f.ctx, f.cancelCtx = context.WithCancel(ctx)
 	f.stopChan = make(chan struct{}, 1)
-	f.registry = NewRegistry(f.cli, f.namespaces, f.conf.NodeName)
+	f.registry = NewRegistry(f.CLI, f.namespaces, f.conf.NodeName)
 	f.manager = manager
 	cache, err := lru.New(5000)
 	if err != nil {
@@ -152,42 +158,9 @@ func (f *ProcessFinder) analyzeProcesses() error {
 		return err
 	}
 
-	result := make([]base.DetectedProcess, 0)
+	result := make([]api.DetectedProcess, 0)
 	for _, p := range processes {
-		createTime, err := p.CreateTime()
-		if err != nil {
-			continue
-		}
-		processCahceKey := fmt.Sprintf("%d_%d", p.Pid, createTime)
-		cachedProcesses, exist := f.processCache.Get(processCahceKey)
-		if exist {
-			for _, pro := range cachedProcesses.([]*Process) {
-				result = append(result, pro)
-			}
-			continue
-		}
-
-		cgroup, err := f.getProcessCGroup(p.Pid)
-		if err != nil {
-			continue
-		}
-
-		c := containers[cgroup]
-		if c == nil {
-			continue
-		}
-
-		// find process builder
-		ps, err := f.buildProcesses(p, c)
-		if err != nil {
-			log.Warnf("find process builder error for pid: %d, err: %v", p.Pid, err)
-			continue
-		}
-
-		for _, pro := range ps {
-			result = append(result, pro)
-		}
-		f.processCache.Add(processCahceKey, ps)
+		result, _ = f.buildProcess(p, result, containers)
 	}
 
 	if len(result) > 0 {
@@ -196,6 +169,51 @@ func (f *ProcessFinder) analyzeProcesses() error {
 	return nil
 }
 
+func (f *ProcessFinder) buildProcess(p *process.Process, detectedProcesses []api.DetectedProcess,
+	containers map[string]*PodContainer) ([]api.DetectedProcess, bool) {
+	createTime, err := p.CreateTime()
+	if err != nil {
+		return detectedProcesses, false
+	}
+	processCahceKey := fmt.Sprintf("%d_%d", p.Pid, createTime)
+	cachedProcesses, exist := f.processCache.Get(processCahceKey)
+	if exist {
+		for _, pro := range cachedProcesses.([]*Process) {
+			detectedProcesses = append(detectedProcesses, pro)
+		}
+		return detectedProcesses, true
+	}
+
+	cgroups, err := f.getProcessCGroup(p.Pid)
+	if err != nil {
+		return detectedProcesses, false
+	}
+
+	var c *PodContainer
+	for _, cgroup := range cgroups {
+		if cc := containers[cgroup]; cc != nil {
+			c = cc
+			break
+		}
+	}
+	if c == nil {
+		return detectedProcesses, false
+	}
+
+	// find process builder
+	ps, err := f.buildProcesses(p, c)
+	if err != nil {
+		log.Warnf("find process builder error for pid: %d, err: %v", p.Pid, err)
+		return detectedProcesses, false
+	}
+
+	for _, pro := range ps {
+		detectedProcesses = append(detectedProcesses, pro)
+	}
+	f.processCache.Add(processCahceKey, ps)
+	return detectedProcesses, true
+}
+
 func (f *ProcessFinder) buildProcesses(p *process.Process, pc *PodContainer) ([]*Process, error) {
 	// find builder
 	builders := make([]*ProcessBuilder, 0)
@@ -231,6 +249,10 @@ func (f *ProcessFinder) buildProcesses(p *process.Process, pc *PodContainer) ([]
 		if err != nil {
 			return nil, err
 		}
+		// adding the cluster name into the service name
+		if f.clusterName != "" && !strings.Contains(entity.ServiceName, "::") {
+			entity.ServiceName = fmt.Sprintf("%s::%s", f.clusterName, entity.ServiceName)
+		}
 		processes = append(processes, NewProcess(p, cmdline, pc, entity))
 	}
 
@@ -244,30 +266,40 @@ func (f *ProcessFinder) buildEntity(err error, ps *process.Process, pc *PodConta
 	return renderTemplate(entity, ps, pc, f)
 }
 
-func (f *ProcessFinder) getProcessCGroup(pid int32) (string, error) {
+func (f *ProcessFinder) getProcessCGroup(pid int32) ([]string, error) {
 	processCgroupFilePath := host.GetFileInHost(fmt.Sprintf("/proc/%d/cgroup", pid))
 	cgroupFile, err := os.Open(processCgroupFilePath)
 	if err != nil {
-		return "", err
+		return nil, err
 	}
 	defer cgroupFile.Close()
 
+	cache := make(map[string]bool)
 	scanner := bufio.NewScanner(cgroupFile)
 	for scanner.Scan() {
 		infos := strings.Split(scanner.Text(), ":")
 		if len(infos) < 3 {
 			continue
 		}
-		// find by most common type of cgroup
-		if infos[1] == "memory" {
-			lastPath := strings.LastIndex(infos[2], "/")
-			if lastPath > 1 && lastPath != len(infos[2])-1 {
-				return infos[2][lastPath+1:], nil
+		lastPath := strings.LastIndex(infos[2], "/")
+		if lastPath > 1 && lastPath != len(infos[2])-1 {
+			path := infos[2][lastPath+1:]
+			// adapt Kubepod
+			// ex: cri-containerd-7dae778c37bd1204677518f1032bbecf01f5c41878ea7bd370021263417cc626.scope
+			if kubepod := kubepodsRegex.FindStringSubmatch(path); len(kubepod) >= 1 {
+				path = kubepod[1]
 			}
-			return "", nil
+			cache[path] = true
 		}
 	}
-	return "", fmt.Errorf("no memory config")
+	if len(cache) == 0 {
+		return nil, fmt.Errorf("no cgroups")
+	}
+	result := make([]string, 0)
+	for k := range cache {
+		result = append(result, k)
+	}
+	return result, nil
 }
 
 func (f *ProcessFinder) Stop() error {
@@ -280,13 +312,13 @@ func (f *ProcessFinder) DetectType() api.ProcessDetectType {
 	return api.Kubernetes
 }
 
-func (f *ProcessFinder) ValidateProcessIsSame(p1, p2 base.DetectedProcess) bool {
+func (f *ProcessFinder) ValidateProcessIsSame(p1, p2 api.DetectedProcess) bool {
 	k1 := p1.(*Process)
 	k2 := p2.(*Process)
 	return p1.Pid() == p2.Pid() && k1.cmd == k2.cmd && p1.Entity().SameWith(p2.Entity())
 }
 
-func (f *ProcessFinder) BuildNecessaryProperties(ps base.DetectedProcess) []*commonv3.KeyStringValuePair {
+func (f *ProcessFinder) BuildNecessaryProperties(ps api.DetectedProcess) []*commonv3.KeyStringValuePair {
 	return []*commonv3.KeyStringValuePair{
 		{
 			Key:   "support_ebpf_profiling",
@@ -295,7 +327,7 @@ func (f *ProcessFinder) BuildNecessaryProperties(ps base.DetectedProcess) []*com
 	}
 }
 
-func (f *ProcessFinder) BuildEBPFProcess(ctx *base.BuildEBPFProcessContext, ps base.DetectedProcess) *v3.EBPFProcessProperties {
+func (f *ProcessFinder) BuildEBPFProcess(ctx *base.BuildEBPFProcessContext, ps api.DetectedProcess) *v3.EBPFProcessProperties {
 	k8sProcess := &v3.EBPFKubernetesProcessMetadata{}
 	k8sProcess.Pid = ps.Pid()
 	k8sProcess.Entity = &v3.EBPFProcessEntityMetadata{
@@ -338,7 +370,7 @@ func (f *ProcessFinder) BuildEBPFProcess(ctx *base.BuildEBPFProcessContext, ps b
 	return properties
 }
 
-func (f *ProcessFinder) ParseProcessID(ps base.DetectedProcess, downstream *v3.EBPFProcessDownstream) string {
+func (f *ProcessFinder) ParseProcessID(ps api.DetectedProcess, downstream *v3.EBPFProcessDownstream) string {
 	if downstream.GetK8SProcess() == nil {
 		return ""
 	}
@@ -348,3 +380,32 @@ func (f *ProcessFinder) ParseProcessID(ps base.DetectedProcess, downstream *v3.E
 	}
 	return ""
 }
+
+func (f *ProcessFinder) ShouldMonitor(pid int32) bool {
+	pidList, err := process.Pids()
+	if err != nil {
+		return false
+	}
+	pidExist := false
+	for _, p := range pidList {
+		if p == pid {
+			pidExist = true
+			break
+		}
+	}
+	if !pidExist {
+		return false
+	}
+
+	newProcess, err := process.NewProcess(pid)
+	if err != nil {
+		return false
+	}
+	// analyze the process needs to be monitored
+	processes, monitor := f.buildProcess(newProcess, nil, f.registry.BuildPodContainers())
+	if !monitor || len(processes) == 0 {
+		return false
+	}
+	f.manager.AddDetectedProcess(processes)
+	return true
+}
