@@ -33,6 +33,9 @@ import (
 
 var log = logger.GetLogger("process", "finder")
 
+// when there have process listener been registered, how often to confrim(recheck) the processes
+var processListenerRecheckInterval = time.Minute
+
 // ProcessManager means Manage all Process
 type ProcessManager struct {
 	moduleManager *module.Manager
@@ -45,6 +48,8 @@ type ProcessManager struct {
 type ProcessManagerWithFinder struct {
 	*ProcessManager
 	finderType api.ProcessDetectType
+
+	lastSync []api.DetectedProcess
 }
 
 func NewProcessManager(ctx context.Context, moduleManager *module.Manager,
@@ -65,7 +70,7 @@ func NewProcessManager(ctx context.Context, moduleManager *module.Manager,
 	}
 
 	// start new storage
-	storage, err := NewProcessStorage(ctx, moduleManager, reportInterval, propertiesReportFactor, fsList)
+	storage, err := NewProcessStorage(ctx, moduleManager, reportInterval, propertiesReportFactor, fsList, processListenerRecheckInterval)
 	if err != nil {
 		return nil, err
 	}
@@ -114,10 +119,54 @@ func (p *ProcessManagerWithFinder) GetModuleManager() *module.Manager {
 	return p.moduleManager
 }
 
-func (p *ProcessManagerWithFinder) SyncAllProcessInFinder(processes []base.DetectedProcess) {
+func (p *ProcessManagerWithFinder) SyncAllProcessInFinder(processes []api.DetectedProcess) {
 	p.storage.SyncAllProcessInFinder(p.finderType, processes)
+	p.lastSync = processes
+}
+
+func (p *ProcessManagerWithFinder) AddDetectedProcess(processes []api.DetectedProcess) {
+	if len(p.lastSync) == 0 {
+		p.SyncAllProcessInFinder(processes)
+		p.lastSync = processes
+		return
+	}
+	// fetch existing process, add the new processes, finally, re-sync
+	detectedProcesses := make([]api.DetectedProcess, 0, len(processes)+len(p.lastSync))
+	detectedProcesses = append(detectedProcesses, p.lastSync...)
+	detectedProcesses = append(detectedProcesses, processes...)
+	p.SyncAllProcessInFinder(detectedProcesses)
+}
+
+func (m *ProcessManager) GetAllProcesses() []api.ProcessInterface {
+	return m.storage.GetAllProcesses()
 }
 
 func (m *ProcessManager) FindProcessByID(processID string) api.ProcessInterface {
 	return m.storage.FindProcessByID(processID)
 }
+
+func (m *ProcessManager) FindProcessByPID(pid int32) []api.ProcessInterface {
+	return m.storage.FindProcessByPID(pid)
+}
+
+func (m *ProcessManager) FindAllRegisteredProcesses() []api.ProcessInterface {
+	return m.storage.FindAllRegisteredProcesses()
+}
+
+func (m *ProcessManager) AddListener(listener api.ProcessListener) {
+	m.storage.AddListener(listener)
+}
+
+func (m *ProcessManager) DeleteListener(listener api.ProcessListener) {
+	m.storage.DeleteListener(listener)
+}
+
+func (m *ProcessManager) ShouldMonitor(pid int32) bool {
+	monitor := false
+	for _, finder := range m.finders {
+		if finder.ShouldMonitor(pid) {
+			monitor = true
+		}
+	}
+	return monitor
+}
