@@ -38,6 +38,12 @@ type ProcessStorage struct {
 	processes map[api.ProcessDetectType][]*ProcessContext
 	mutex     sync.Mutex
 
+	// process listeners
+	listeners               []api.ProcessListener
+	eventQueue              chan *processEvent
+	initListenQueue         chan api.ProcessListener
+	listenerRecheckInterval time.Duration
+
 	// working with backend
 	reportInterval         time.Duration
 	propertiesReportFactor int
@@ -51,8 +57,8 @@ type ProcessStorage struct {
 	cancel context.CancelFunc
 }
 
-func NewProcessStorage(ctx context.Context, moduleManager *module.Manager,
-	reportInterval time.Duration, propertiesReportFactor int, finderList []base.ProcessFinder) (*ProcessStorage, error) {
+func NewProcessStorage(ctx context.Context, moduleManager *module.Manager, reportInterval time.Duration,
+	propertiesReportFactor int, finderList []base.ProcessFinder, listenerRecheckInterval time.Duration) (*ProcessStorage, error) {
 	data := make(map[api.ProcessDetectType][]*ProcessContext)
 	// working with core module
 	coreOperator := moduleManager.FindModule(core.ModuleName).(core.Operator)
@@ -65,19 +71,23 @@ func NewProcessStorage(ctx context.Context, moduleManager *module.Manager,
 		fs[f.DetectType()] = f
 	}
 	return &ProcessStorage{
-		processes:              data,
-		reportInterval:         reportInterval,
-		propertiesReportFactor: propertiesReportFactor,
-		reportedCount:          0,
-		roverID:                roverID,
-		processClient:          processClient,
-		finders:                fs,
-		ctx:                    ctx,
-		cancel:                 cancel,
+		processes:               data,
+		reportInterval:          reportInterval,
+		propertiesReportFactor:  propertiesReportFactor,
+		eventQueue:              make(chan *processEvent, 100),
+		initListenQueue:         make(chan api.ProcessListener, 100),
+		listenerRecheckInterval: listenerRecheckInterval,
+		reportedCount:           0,
+		roverID:                 roverID,
+		processClient:           processClient,
+		finders:                 fs,
+		ctx:                     ctx,
+		cancel:                  cancel,
 	}, nil
 }
 
 func (s *ProcessStorage) StartReport() {
+	// for report all processes
 	go func() {
 		timeTicker := time.NewTicker(s.reportInterval)
 		for {
@@ -92,6 +102,24 @@ func (s *ProcessStorage) StartReport() {
 			}
 		}
 	}()
+
+	// for start listener
+	go func() {
+		timeTicker := time.NewTicker(s.listenerRecheckInterval)
+		for {
+			select {
+			case <-timeTicker.C:
+				s.notifyToRecheckAllProcesses(s.listeners)
+			case e := <-s.eventQueue:
+				s.consumeProcessEvent(s.listeners, e)
+			case l := <-s.initListenQueue:
+				s.notifyToRecheckAllProcesses([]api.ProcessListener{l})
+			case <-s.ctx.Done():
+				timeTicker.Stop()
+				return
+			}
+		}
+	}()
 }
 
 func (s *ProcessStorage) StopReport() error {
@@ -185,6 +213,7 @@ func (s *ProcessStorage) processesReport(waitReportProcesses []*ProcessContext)
 		return err
 	}
 
+	eventBuilder := s.newProcessEventBuilder(ProcessOperateAdd)
 	for _, waitProcess := range waitReportProcesses {
 		found := false
 		for _, reportedProcess := range processes.GetProcesses() {
@@ -195,6 +224,7 @@ func (s *ProcessStorage) processesReport(waitReportProcesses []*ProcessContext)
 
 			s.updateProcessToUploadSuccess(waitProcess, id)
 			found = true
+			eventBuilder.AddProcess(waitProcess.Pid(), waitProcess)
 			break
 		}
 
@@ -202,10 +232,12 @@ func (s *ProcessStorage) processesReport(waitReportProcesses []*ProcessContext)
 			s.updateProcessToUploadIgnored(waitProcess)
 		}
 	}
+
+	eventBuilder.Send()
 	return nil
 }
 
-func (s *ProcessStorage) SyncAllProcessInFinder(finder api.ProcessDetectType, processes []base.DetectedProcess) {
+func (s *ProcessStorage) SyncAllProcessInFinder(finder api.ProcessDetectType, processes []api.DetectedProcess) {
 	s.mutex.Lock()
 	defer s.mutex.Unlock()
 
@@ -217,7 +249,11 @@ func (s *ProcessStorage) SyncAllProcessInFinder(finder api.ProcessDetectType, pr
 		existingProcessHasFounded[p] = false
 	}
 
+	addProcessBuilder := s.newProcessEventBuilder(ProcessOperateAdd)
 	for _, syncProcess := range processes {
+		if syncProcess == nil {
+			continue
+		}
 		founded := false
 		for _, existingProcess := range existingProcesses {
 			if syncProcess.Pid() == existingProcess.Pid() && syncProcess.Entity().SameWith(existingProcess.Entity()) {
@@ -230,27 +266,38 @@ func (s *ProcessStorage) SyncAllProcessInFinder(finder api.ProcessDetectType, pr
 
 		// if not found in existing processes, need to add this process
 		if !founded {
-			newProcesses = append(newProcesses, s.constructNewProcessContext(finder, syncProcess))
+			processContext := s.constructNewProcessContext(finder, syncProcess)
+			newProcesses = append(newProcesses, processContext)
+			addProcessBuilder.AddProcess(syncProcess.Pid(), newProcesses[len(newProcesses)-1])
 			log.Infof("detected new process: pid: %d, entity: %s", syncProcess.Pid(), syncProcess.Entity())
 		}
 	}
+	addProcessBuilder.Send()
 
 	// log the dead processes
+	eventBuilder := s.newProcessEventBuilder(ProcessOperateDelete)
 	for p, found := range existingProcessHasFounded {
 		if found {
 			continue
 		}
 		log.Infof("the process has been recognized as dead, so deleted. pid: %d, entity: %s, id: %s", p.Pid(), p.Entity(), p.id)
+		eventBuilder.AddProcess(p.Pid(), p)
 	}
 
 	s.processes[finder] = newProcesses
+	eventBuilder.Send()
 }
 
-func (s *ProcessStorage) constructNewProcessContext(finder api.ProcessDetectType, process base.DetectedProcess) *ProcessContext {
+func (s *ProcessStorage) constructNewProcessContext(finder api.ProcessDetectType, process api.DetectedProcess) *ProcessContext {
+	exporsedPorts := make(map[int]bool)
+	for _, p := range process.ExposePorts() {
+		exporsedPorts[p] = true
+	}
 	return &ProcessContext{
 		syncStatus:    NotReport,
 		detectProcess: process,
 		detectType:    finder,
+		exposedPorts:  exporsedPorts,
 	}
 }
 
@@ -268,6 +315,28 @@ func (s *ProcessStorage) updateProcessToUploadIgnored(pc *ProcessContext) {
 	log.Infof("could not found the process id from upstream, pid: %d, entity: %v", pc.Pid(), pc.Entity())
 }
 
+func (s *ProcessStorage) GetAllProcesses() []api.ProcessInterface {
+	result := make([]api.ProcessInterface, 0)
+	for _, processList := range s.processes {
+		for _, p := range processList {
+			result = append(result, p)
+		}
+	}
+	return result
+}
+
+func (s *ProcessStorage) FindAllRegisteredProcesses() []api.ProcessInterface {
+	result := make([]api.ProcessInterface, 0)
+	for _, processList := range s.processes {
+		for _, p := range processList {
+			if p.syncStatus == ReportSuccess {
+				result = append(result, p)
+			}
+		}
+	}
+	return result
+}
+
 func (s *ProcessStorage) FindProcessByID(processID string) api.ProcessInterface {
 	for _, finderProcesses := range s.processes {
 		for _, p := range finderProcesses {
@@ -278,3 +347,99 @@ func (s *ProcessStorage) FindProcessByID(processID string) api.ProcessInterface
 	}
 	return nil
 }
+
+func (s *ProcessStorage) FindProcessByPID(pid int32) []api.ProcessInterface {
+	result := make([]api.ProcessInterface, 0)
+	for _, finderProcesses := range s.processes {
+		for _, p := range finderProcesses {
+			if p.Pid() == pid {
+				result = append(result, p)
+			}
+		}
+	}
+	return result
+}
+
+func (s *ProcessStorage) AddListener(listener api.ProcessListener) {
+	s.listeners = append(s.listeners, listener)
+	s.initListenQueue <- listener
+}
+
+func (s *ProcessStorage) DeleteListener(listener api.ProcessListener) {
+	result := make([]api.ProcessListener, 0)
+	for _, l := range s.listeners {
+		if l != listener {
+			result = append(result, l)
+		}
+	}
+	s.listeners = result
+}
+
+type ProcessOperate int
+
+const (
+	ProcessOperateAdd    = 1
+	ProcessOperateDelete = 2
+)
+
+type processEventBuilder struct {
+	processes map[int32][]api.ProcessInterface
+	operate   ProcessOperate
+	storage   *ProcessStorage
+}
+
+func (s *ProcessStorage) newProcessEventBuilder(operate ProcessOperate) *processEventBuilder {
+	return &processEventBuilder{
+		processes: make(map[int32][]api.ProcessInterface),
+		operate:   operate,
+		storage:   s,
+	}
+}
+
+func (p *processEventBuilder) AddProcess(pid int32, pi api.ProcessInterface) {
+	ps := p.processes[pid]
+	ps = append(ps, pi)
+	p.processes[pid] = ps
+}
+
+func (p *processEventBuilder) Send() {
+	for pid, processes := range p.processes {
+		p.storage.eventQueue <- &processEvent{
+			pid:       pid,
+			processes: processes,
+			operate:   p.operate,
+		}
+	}
+}
+
+type processEvent struct {
+	pid       int32
+	processes []api.ProcessInterface
+	operate   ProcessOperate
+}
+
+func (s *ProcessStorage) consumeProcessEvent(listeners []api.ProcessListener, e *processEvent) {
+	for _, l := range listeners {
+		if e.operate == ProcessOperateAdd {
+			l.AddNewProcess(e.pid, e.processes)
+		} else {
+			l.RemoveProcess(e.pid, e.processes)
+		}
+	}
+}
+
+func (s *ProcessStorage) notifyToRecheckAllProcesses(listeners []api.ProcessListener) {
+	if len(listeners) == 0 {
+		return
+	}
+	// build all processes
+	events := s.newProcessEventBuilder(ProcessOperateAdd)
+	for _, pcs := range s.processes {
+		for _, pc := range pcs {
+			events.AddProcess(pc.Pid(), pc)
+		}
+	}
+	for _, l := range listeners {
+		l.RecheckAllProcesses(events.processes)
+	}
+}
